unit YarrowModule;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MPYarrow;

type
  TdmYarrow = class(TDataModule)
    MPYarrow1: TMPYarrow;
    procedure DataModuleCreate(Sender: TObject);
    procedure MPYarrow1Reseeded(Sender: TObject; Slow: Boolean);
    procedure MPYarrow1Reseeding(Sender: TObject; Slow: Boolean);
    procedure DataModuleDestroy(Sender: TObject);
  private
    FSlowReseeds: Cardinal;
    FFastReseeds: Cardinal;
    FReseedThreads: Cardinal;
    FOnReseed: TNotifyEvent;
    FOnReseeding: TNotifyEvent;
    procedure ApplicationMessage(var Msg: TMsg; var Handled: Boolean);
    procedure SetOnReseed(const Value: TNotifyEvent);
    procedure SetOnReseeding(const Value: TNotifyEvent);
    { Private declarations }
  public
    procedure ForceSlowReseed(const Buf; Count: Integer);
    property FastReseeds: Cardinal read FFastReseeds;
    property SlowReseeds: Cardinal read FSlowReseeds;
    property ReseedThreads: Cardinal read FReseedThreads;
    property OnReseed: TNotifyEvent read FOnReseed write SetOnReseed;  
    property OnReseeding: TNotifyEvent read FOnReseeding write SetOnReseeding;
  end;

var
  dmYarrow: TdmYarrow;

implementation

uses
  SecUtils;

{$R *.DFM}

procedure TdmYarrow.ApplicationMessage(var Msg: TMsg;
  var Handled: Boolean);
begin
  MPYarrow1.Accumulate(Msg,SizeOf(Msg),1,0);
end;

procedure TdmYarrow.DataModuleCreate(Sender: TObject);
begin
  if Assigned(dmYarrow) and (dmYarrow <> Self) then
    raise Exception.Create('There can be only one instance of TdmYarrow per application');
  Application.OnMessage := ApplicationMessage;
  MPYarrow1.ForceSlowReseed(nil^,0,1000);
end;

procedure TdmYarrow.SetOnReseed(const Value: TNotifyEvent);
begin
  FOnReseed := Value;
end;

procedure TdmYarrow.MPYarrow1Reseeded(Sender: TObject; Slow: Boolean);
begin
  if FReseedThreads > 0 then
    Dec(FReseedThreads);
  if Slow then
    Inc(FSlowReseeds)
  else
    Inc(FFastReseeds);
  if Assigned(FOnReseed) then
    FOnReseed(Self);
end;

procedure TdmYarrow.ForceSlowReseed(const Buf; Count: Integer);
begin
  MPYarrow1.Accumulate(Buf,Count,0,Count shr 1);
  MPYarrow1.ForceSlowReseed(Buf,Count,500);
end;

procedure TdmYarrow.MPYarrow1Reseeding(Sender: TObject; Slow: Boolean);
begin
  Inc(FReseedThreads);
  if Assigned(FonReseeding) then
    FOnReseeding(Self);
end;

procedure TdmYarrow.SetOnReseeding(const Value: TNotifyEvent);
begin
  FOnReseeding := Value;
end;

procedure TdmYarrow.DataModuleDestroy(Sender: TObject);
begin
  dmYarrow := nil;
end;

end.
